import logging
import click
import json
import os

from stactools.sentinel2.stac import create_item

logger = logging.getLogger(__name__)


def create_sentinel2_command(cli):
    @cli.group('sentinel2',
               short_help=("Commands for working with sentinel2 data"))
    def sentinel2():
        pass

    @sentinel2.command(
        'create-item',
        short_help='Convert a Sentinel2 L2A granule into a STAC item')
    @click.argument('src')
    @click.argument('dst')
    @click.option(
        '-p',
        '--providers',
        help='Path to JSON file containing array of additional providers')
    def create_item_command(src, dst, providers):
        """Creates a STAC Item for a given Sentinel 2 granule

        SRC is the path to the granule
        DST is directory that a STAC Item JSON file will be created
        in. This will have a filename that matches the ID, which will
        be derived from the Sentinel 2 metadata.
        """
        additional_providers = None
        if providers is not None:
            with open(providers) as f:
                additional_providers = json.load(f)

        item = create_item(src, additional_providers=additional_providers)

        item_path = os.path.join(dst, '{}.json'.format(item.id))
        item.set_self_href(item_path)

        item.save_object()

    return sentinel2



from datetime import datetime
import re
from typing import Any, Dict, List, Optional

from shapely.geometry import mapping, Polygon
import pystac
from pystac.utils import str_to_datetime

from stactools.core.io import ReadHrefModifier
from stactools.core.io.xml import XmlElement
from stactools.core.utils import map_opt
from stactools.sentinel2.constants import PRODUCT_METADATA_ASSET_KEY


class ProductMetadataError(Exception):
    pass


class ProductMetadata:
    def __init__(
            self,
            href,
            read_href_modifier: Optional[ReadHrefModifier] = None) -> None:
        self.href = href
        self._root = XmlElement.from_file(href, read_href_modifier)

        product_info_node = self._root.find('n1:General_Info/Product_Info')
        if product_info_node is None:
            raise ProductMetadataError(
                f"Cannot find product info node for product metadata at {self.href}"
            )
        self.product_info_node = product_info_node

        datatake_node = self.product_info_node.find('Datatake')
        if datatake_node is None:
            raise ProductMetadataError(
                f"Cannot find Datatake node in product metadata at {self.href}"
            )
        self.datatake_node = datatake_node

        granule_node = self.product_info_node.find(
            'Product_Organisation/Granule_List/Granule')
        if granule_node is None:
            raise ProductMetadataError(
                f'Cannot find granule node in product metadata at {self.href}')
        self.granule_node = granule_node

        reflectance_conversion_node = self._root.find(
            'n1:General_Info/Product_Image_Characteristics/Reflectance_Conversion'
        )
        if reflectance_conversion_node is None:
            raise ProductMetadataError(
                f"Could not find reflectance conversion node in product metadata at {self.href}"
            )
        self.reflectance_conversion_node = reflectance_conversion_node

        qa_node = self._root.find('n1:Quality_Indicators_Info')
        if qa_node is None:
            raise ProductMetadataError(
                f"Could not find QA node in product metadata at {self.href}")
        self.qa_node = qa_node

        def _get_geometries():
            geometric_info = self._root.find('n1:Geometric_Info')
            footprint_text = geometric_info.find_text(
                'Product_Footprint/Product_Footprint/Global_Footprint/EXT_POS_LIST'
            )
            if footprint_text is None:
                ProductMetadataError(
                    f'Cannot parse footprint from product metadata at {self.href}'
                )

            # Some coordinates contain a 0 z dimension for some reason;
            # filter those out
            footprint_coords = [
                float(c) for c in footprint_text.split(' ') if c and c != '0'
            ]
            footprint_points = [
                p[::-1] for p in list(zip(*[iter(footprint_coords)] * 2))
            ]
            footprint_polygon = Polygon(footprint_points)
            geometry = mapping(footprint_polygon)
            bbox = footprint_polygon.bounds

            return (bbox, geometry)

        self.bbox, self.geometry = _get_geometries()

    @property
    def product_id(self) -> str:
        result = self.product_info_node.find_text('PRODUCT_URI')
        if result is None:
            raise ValueError(
                'Cannot determine product ID using product metadata '
                f'at {self.href}')
        else:
            return result

    @property
    def datetime(self) -> datetime:
        time = self.product_info_node.find_text('PRODUCT_START_TIME')
        if time is None:
            raise ValueError(
                'Cannot determine product start time using product metadata '
                f'at {self.href}')
        else:
            return str_to_datetime(time)

    @property
    def image_media_type(self) -> str:
        if self.granule_node.get_attr('imageFormat') == 'GeoTIFF':
            return pystac.MediaType.COG
        else:
            return pystac.MediaType.JPEG2000

    @property
    def image_paths(self) -> List[str]:
        extension = '.tif' if self.image_media_type == pystac.MediaType.COG else '.jp2'

        return [
            f'{x.text}{extension}'
            for x in self.granule_node.findall('IMAGE_FILE')
        ]

    @property
    def relative_orbit(self) -> Optional[int]:
        return map_opt(int,
                       self.datatake_node.find_text('SENSING_ORBIT_NUMBER'))

    @property
    def orbit_state(self) -> Optional[str]:
        return self.datatake_node.find_text('SENSING_ORBIT_DIRECTION')

    @property
    def platform(self) -> Optional[str]:
        return self.datatake_node.find_text('SPACECRAFT_NAME')

    @property
    def mgrs_tile(self) -> Optional[str]:
        m = re.search(r'_T(\d{2}[a-zA-Z]{3})_', self.href)
        return None if m is None else m.group(1)

    @property
    def metadata_dict(self) -> Dict[str, Any]:
        result = {
            's2:product_uri':
            self.product_id,
            's2:generation_time':
            self.product_info_node.find_text('GENERATION_TIME'),
            's2:processing_baseline':
            self.product_info_node.find_text('PROCESSING_BASELINE'),
            's2:product_type':
            self.product_info_node.find_text('PRODUCT_TYPE'),
            's2:datatake_id':
            self.datatake_node.get_attr('datatakeIdentifier'),
            's2:datatake_type':
            self.datatake_node.find_text('DATATAKE_TYPE'),
            's2:datastrip_id':
            self.granule_node.get_attr('datastripIdentifier'),
            's2:granule_id':
            self.granule_node.get_attr('granuleIdentifier'),
            's2:mgrs_tile':
            self.mgrs_tile,
            's2:reflectance_conversion_factor':
            map_opt(float, self.reflectance_conversion_node.find_text('U'))
        }

        return {k: v for k, v in result.items() if v is not None}

    def create_asset(self):
        asset = pystac.Asset(href=self.href,
                             media_type=pystac.MediaType.XML,
                             roles=['metadata'])
        return (PRODUCT_METADATA_ASSET_KEY, asset)



import logging
import os
import re
from typing import Dict, List, Optional, Tuple

import pystac
from pystac.extensions.sat import OrbitState

from stactools.core.io import ReadHrefModifier
from stactools.core.projection import transform_from_bbox
from stactools.sentinel2.safe_manifest import SafeManifest
from stactools.sentinel2.product_metadata import ProductMetadata
from stactools.sentinel2.granule_metadata import GranuleMetadata
from stactools.sentinel2.utils import extract_gsd
from stactools.sentinel2.constants import (
    BANDS_TO_RESOLUTIONS, DATASTRIP_METADATA_ASSET_KEY, SENTINEL_PROVIDER,
    SENTINEL_LICENSE, SENTINEL_BANDS, SENTINEL_INSTRUMENTS,
    SENTINEL_CONSTELLATION, INSPIRE_METADATA_ASSET_KEY)

logger = logging.getLogger(__name__)


def create_item(
        granule_href: str,
        additional_providers: Optional[List[pystac.Provider]] = None,
        read_href_modifier: Optional[ReadHrefModifier] = None) -> pystac.Item:
    """Create a STC Item from a Sentinel 2 granule.

    Arguments:
        granule_href: The HREF to the granule. This is expected to be a path
            to a SAFE archive, e.g. : https://sentinel2l2a01.blob.core.windows.net/sentinel2-l2/01/C/CV/2016/03/27/S2A_MSIL2A_20160327T204522_N0212_R128_T01CCV_20210214T042702.SAFE
        additional_providers: Optional list of additional providers to set into the Item
        read_href_modifier: A function that takes an HREF and returns a modified HREF.
            This can be used to modify a HREF to make it readable, e.g. appending
            an Azure SAS token or creating a signed URL.

    Returns:
        pystac.Item: An item representing the Sentinel 2 scene
    """ # noqa

    safe_manifest = SafeManifest(granule_href, read_href_modifier)

    product_metadata = ProductMetadata(safe_manifest.product_metadata_href,
                                       read_href_modifier)
    granule_metadata = GranuleMetadata(safe_manifest.granule_metadata_href,
                                       read_href_modifier)

    item = pystac.Item(id=product_metadata.product_id,
                       geometry=product_metadata.geometry,
                       bbox=product_metadata.bbox,
                       datetime=product_metadata.datetime,
                       properties={})

    # --Common metadata--

    item.common_metadata.providers = [SENTINEL_PROVIDER]

    if additional_providers is not None:
        item.common_metadata.providers.extend(additional_providers)

    item.common_metadata.platform = product_metadata.platform
    item.common_metadata.constellation = SENTINEL_CONSTELLATION
    item.common_metadata.instruments = SENTINEL_INSTRUMENTS

    # --Extensions--

    # eo

    item.ext.enable('eo')
    item.ext.eo.cloud_cover = granule_metadata.cloudiness_percentage

    # sat

    item.ext.enable('sat')
    item.ext.sat.orbit_state = OrbitState(product_metadata.orbit_state.lower())
    item.ext.sat.relative_orbit = product_metadata.relative_orbit

    # proj
    item.ext.enable('projection')
    item.ext.projection.epsg = granule_metadata.epsg
    if item.ext.projection.epsg is None:
        raise ValueError(
            f'Could not determine EPSG code for {granule_href}; which is required.'
        )

    # s2 properties
    item.properties.update({
        **product_metadata.metadata_dict,
        **granule_metadata.metadata_dict
    })

    # --Assets--

    # Metadata

    item.add_asset(*safe_manifest.create_asset())
    item.add_asset(*product_metadata.create_asset())
    item.add_asset(*granule_metadata.create_asset())
    item.add_asset(
        INSPIRE_METADATA_ASSET_KEY,
        pystac.Asset(href=safe_manifest.inspire_metadata_href,
                     media_type=pystac.MediaType.XML,
                     roles=['metadata']))
    item.add_asset(
        DATASTRIP_METADATA_ASSET_KEY,
        pystac.Asset(href=safe_manifest.datastrip_metadata_href,
                     media_type=pystac.MediaType.XML,
                     roles=['metadata']))

    # Image assets
    proj_bbox = granule_metadata.proj_bbox

    image_assets = dict([
        image_asset_from_href(os.path.join(granule_href, image_path), item,
                              granule_metadata.resolution_to_shape, proj_bbox,
                              product_metadata.image_media_type)
        for image_path in product_metadata.image_paths
    ])

    for key, asset in image_assets.items():
        assert key not in item.assets
        item.add_asset(key, asset)

    # Thumbnail

    if safe_manifest.thumbnail_href is not None:
        item.add_asset(
            "preview",
            pystac.Asset(href=safe_manifest.thumbnail_href,
                         media_type=pystac.MediaType.COG,
                         roles=['thumbnail']))

    # --Links--

    item.links.append(SENTINEL_LICENSE)

    return item


def image_asset_from_href(
        asset_href: str,
        item: pystac.Item,
        resolution_to_shape: Dict[int, Tuple[int, int]],
        proj_bbox: List[float],
        media_type: Optional[str] = None) -> Tuple[str, pystac.Asset]:
    logger.debug(f'Creating asset for image {asset_href}')

    _, ext = os.path.splitext(asset_href)
    if media_type is not None:
        asset_media_type = media_type
    else:
        if ext.lower() == '.jp2':
            asset_media_type = pystac.MediaType.JPEG2000
        elif ext.lower() in ['.tiff', '.tif']:
            asset_media_type = pystac.MediaType.GEOTIFF
        else:
            raise Exception(
                f'Must supply a media type for asset : {asset_href}')

    # Handle preview image

    if '_PVI' in asset_href:
        asset = pystac.Asset(href=asset_href,
                             media_type=asset_media_type,
                             title='True color preview',
                             roles=['data'])
        item.ext.eo.set_bands([
            SENTINEL_BANDS['B04'], SENTINEL_BANDS['B03'], SENTINEL_BANDS['B02']
        ], asset)
        return ('preview', asset)

    # Extract gsd and proj info
    filename_gsd = extract_gsd(asset_href)
    shape = list(resolution_to_shape[int(filename_gsd)])
    transform = transform_from_bbox(proj_bbox, shape)

    def set_asset_properties(asset: pystac.Asset,
                             band_gsd: Optional[int] = None):
        if band_gsd:
            item.common_metadata.set_gsd(band_gsd, asset)
        item.ext.projection.set_shape(shape, asset)
        item.ext.projection.set_bbox(proj_bbox, asset)
        item.ext.projection.set_transform(transform, asset)

    # Handle band image

    band_id_search = re.search(r'_(B\w{2})', asset_href)
    if band_id_search is not None:
        band_id, href_res = os.path.splitext(asset_href)[0].split('_')[-2:]
        band = SENTINEL_BANDS[band_id]

        # Get the asset resolution from the file name.
        # If the asset resolution is the band GSD, then
        # include the gsd information for that asset. Otherwise,
        # do not include the GSD information in the asset
        # as this may be confusing for users given that the
        # raster spatial resolution and gsd will differ.
        # See https://github.com/radiantearth/stac-spec/issues/1096
        asset_res = int(href_res.replace('m', ''))
        band_gsd: Optional[int] = None
        if asset_res == BANDS_TO_RESOLUTIONS[band_id][0]:
            asset_key = band_id
            band_gsd = asset_res
        else:
            # If this isn't the default resolution, use the raster
            # resolution in the asset key.
            # TODO: Use the raster extension and spatial_resolution
            # property to encode the spatial resolution of all assets.
            asset_key = f'{band_id}_{asset_res}m'

        asset = pystac.Asset(href=asset_href,
                             media_type=asset_media_type,
                             title=f'{band.description} - {href_res}',
                             roles=['data'])

        item.ext.eo.set_bands([SENTINEL_BANDS[band_id]], asset)
        set_asset_properties(asset, band_gsd=band_gsd)
        return (asset_key, asset)

    # Handle auxiliary images

    if '_TCI_' in asset_href:
        # True color
        asset = pystac.Asset(href=asset_href,
                             media_type=asset_media_type,
                             title='True color image',
                             roles=['data'])
        item.ext.eo.set_bands([
            SENTINEL_BANDS['B04'], SENTINEL_BANDS['B03'], SENTINEL_BANDS['B02']
        ], asset)
        set_asset_properties(asset)
        return (f'visual-{asset_href[-7:-4]}', asset)

    if '_AOT_' in asset_href:
        # Aerosol
        asset = pystac.Asset(href=asset_href,
                             media_type=asset_media_type,
                             title='Aerosol optical thickness (AOT)',
                             roles=['data'])
        set_asset_properties(asset)
        return (f'AOT-{asset_href[-7:-4]}', asset)

    if '_WVP_' in asset_href:
        # Water vapor
        asset = pystac.Asset(href=asset_href,
                             media_type=asset_media_type,
                             title='Water vapour (WVP)',
                             roles=['data'])
        set_asset_properties(asset)
        return (f'WVP-{asset_href[-7:-4]}', asset)

    if '_SCL_' in asset_href:
        # Classification map
        asset = pystac.Asset(href=asset_href,
                             media_type=asset_media_type,
                             title='Scene classfication map (SCL)',
                             roles=['data'])
        set_asset_properties(asset)
        return (f'SCL-{asset_href[-7:-4]}', asset)

    raise ValueError(f'Unexpected asset: {asset_href}')



from typing import Dict, List, Optional, Tuple

import pystac

from stactools.core.io import ReadHrefModifier
from stactools.core.io.xml import XmlElement
from stactools.core.utils import map_opt
from stactools.sentinel2.constants import GRANULE_METADATA_ASSET_KEY


class GranuleMetadataError(Exception):
    pass


class GranuleMetadata:
    def __init__(self,
                 href,
                 read_href_modifier: Optional[ReadHrefModifier] = None):
        self.href = href

        self._root = XmlElement.from_file(href, read_href_modifier)

        geocoding_node = self._root.find('n1:Geometric_Info/Tile_Geocoding')
        if geocoding_node is None:
            raise GranuleMetadataError(
                f"Cannot find geocoding node in {self.href}")
        self._geocoding_node = geocoding_node

        tile_angles_node = self._root.find('n1:Geometric_Info/Tile_Angles')
        if tile_angles_node is None:
            raise GranuleMetadataError(
                f"Cannot find tile angles node in {self.href}")
        self._tile_angles_node = tile_angles_node

        self._viewing_angle_nodes = self._tile_angles_node.findall(
            'Mean_Viewing_Incidence_Angle_List/Mean_Viewing_Incidence_Angle')

        self._image_content_node = self._root.find(
            'n1:Quality_Indicators_Info/Image_Content_QI')

        self.resolution_to_shape: Dict[int, Tuple[int, int]] = {}
        for size_node in self._geocoding_node.findall("Size"):
            res = size_node.get_attr("resolution")
            if res is None:
                raise GranuleMetadataError(
                    'Size element does not have resolution.')
            nrows = map_opt(int, size_node.find_text('NROWS'))
            if nrows is None:
                raise GranuleMetadataError(
                    f'Could not get rows from size for resolution {res}')
            ncols = map_opt(int, size_node.find_text('NCOLS'))
            if ncols is None:
                raise GranuleMetadataError(
                    f'Could not get columns from size for resolution {res}')
            self.resolution_to_shape[int(res)] = (nrows, ncols)

    @property
    def epsg(self) -> Optional[int]:
        epsg_str = self._geocoding_node.find_text('HORIZONTAL_CS_CODE')
        if epsg_str is None:
            return None
        else:
            return int(epsg_str.split(':')[1])

    @property
    def proj_bbox(self) -> List[float]:
        """The bbox of the image in the CRS of the image data"""
        nrows, ncols = self.resolution_to_shape[10]
        geoposition = self._geocoding_node.find('Geoposition')
        if geoposition is None:
            raise GranuleMetadataError(
                f'Cannot find geoposition node in {self.href}')
        ulx = map_opt(float, geoposition.find_text('ULX'))
        if ulx is None:
            raise GranuleMetadataError('Could not get upper left X coordinate')
        uly = map_opt(float, geoposition.find_text('ULY'))
        if uly is None:
            raise GranuleMetadataError('Could not get upper left Y coordinate')

        return [ulx, uly - (10 * nrows), ulx + (10 * ncols), uly]

    @property
    def cloudiness_percentage(self) -> Optional[float]:
        return map_opt(
            float,
            self._image_content_node.find_text('CLOUDY_PIXEL_PERCENTAGE'))

    @property
    def mean_solar_zenith(self) -> Optional[float]:
        return map_opt(
            float,
            self._tile_angles_node.find_text('Mean_Sun_Angle/ZENITH_ANGLE'))

    @property
    def mean_solar_azimuth(self) -> Optional[float]:
        return map_opt(
            float,
            self._tile_angles_node.find_text('Mean_Sun_Angle/AZIMUTH_ANGLE'))

    @property
    def metadata_dict(self):
        image_content_properties = {
            's2:degraded_msi_data_percentage':
            map_opt(
                float,
                self._image_content_node.find_text(
                    'DEGRADED_MSI_DATA_PERCENTAGE')),
            's2:nodata_pixel_percentage':
            map_opt(
                float,
                self._image_content_node.find_text('NODATA_PIXEL_PERCENTAGE')),
            's2:saturated_defective_pixel_percentage':
            map_opt(
                float,
                self._image_content_node.find_text(
                    'SATURATED_DEFECTIVE_PIXEL_PERCENTAGE')),
            's2:dark_features_percentage':
            map_opt(
                float,
                self._image_content_node.find_text(
                    'DARK_FEATURES_PERCENTAGE')),
            's2:cloud_shadow_percentage':
            map_opt(
                float,
                self._image_content_node.find_text('CLOUD_SHADOW_PERCENTAGE')),
            's2:vegetation_percentage':
            map_opt(
                float,
                self._image_content_node.find_text('VEGETATION_PERCENTAGE')),
            's2:not_vegetated_percentage':
            map_opt(
                float,
                self._image_content_node.find_text(
                    'NOT_VEGETATED_PERCENTAGE')),
            's2:water_percentage':
            map_opt(float,
                    self._image_content_node.find_text('WATER_PERCENTAGE')),
            's2:unclassified_percentage':
            map_opt(
                float,
                self._image_content_node.find_text('UNCLASSIFIED_PERCENTAGE')),
            's2:medium_proba_clouds_percentage':
            map_opt(
                float,
                self._image_content_node.find_text(
                    'MEDIUM_PROBA_CLOUDS_PERCENTAGE')),
            's2:high_proba_clouds_percentage':
            map_opt(
                float,
                self._image_content_node.find_text(
                    'HIGH_PROBA_CLOUDS_PERCENTAGE')),
            's2:thin_cirrus_percentage':
            map_opt(
                float,
                self._image_content_node.find_text('THIN_CIRRUS_PERCENTAGE')),
            's2:snow_ice_percentage':
            map_opt(float,
                    self._image_content_node.find_text('SNOW_ICE_PERCENTAGE'))
        }

        result = {
            **image_content_properties,
            's2:mean_solar_zenith': self.mean_solar_zenith,
            's2:mean_solar_azimuth': self.mean_solar_azimuth,
        }

        return {k: v for k, v in result.items() if v is not None}

    def create_asset(self):
        asset = pystac.Asset(href=self.href,
                             media_type=pystac.MediaType.XML,
                             roles=['metadata'])
        return (GRANULE_METADATA_ASSET_KEY, asset)



import pystac
from pystac.link import Link
from pystac.extensions.eo import Band

SENTINEL_LICENSE = Link(rel='license',
                        target='https://sentinel.esa.int/documents/' +
                        '247904/690755/Sentinel_Data_Legal_Notice')

SENTINEL_INSTRUMENTS = ['msi']
SENTINEL_CONSTELLATION = 'Sentinel 2'

SENTINEL_PROVIDER = pystac.Provider(
    name='ESA',
    roles=['producer', 'processor', 'licensor'],
    url='https://earth.esa.int/web/guest/home')

SAFE_MANIFEST_ASSET_KEY = "safe-manifest"
INSPIRE_METADATA_ASSET_KEY = "inspire-metadata"
PRODUCT_METADATA_ASSET_KEY = "product-metadata"
GRANULE_METADATA_ASSET_KEY = "granule-metadata"
DATASTRIP_METADATA_ASSET_KEY = "datastrip-metadata"

SENTINEL_BANDS = {
    'B01':
    Band.create(name='B01',
                common_name='coastal',
                description='Band 1 - Coastal aerosol',
                center_wavelength=0.443,
                full_width_half_max=0.027),
    'B02':
    Band.create(name='B02',
                common_name='blue',
                description='Band 2 - Blue',
                center_wavelength=0.490,
                full_width_half_max=0.098),
    'B03':
    Band.create(name='B03',
                common_name='green',
                description='Band 3 - Green',
                center_wavelength=0.560,
                full_width_half_max=0.045),
    'B04':
    Band.create(name='B04',
                common_name='red',
                description='Band 4 - Red',
                center_wavelength=0.665,
                full_width_half_max=0.038),
    'B05':
    Band.create(name='B05',
                common_name='rededge',
                description='Band 5 - Vegetation red edge 1',
                center_wavelength=0.704,
                full_width_half_max=0.019),
    'B06':
    Band.create(name='B06',
                common_name='rededge',
                description='Band 6 - Vegetation red edge 2',
                center_wavelength=0.740,
                full_width_half_max=0.018),
    'B07':
    Band.create(name='B07',
                common_name='rededge',
                description='Band 7 - Vegetation red edge 3',
                center_wavelength=0.783,
                full_width_half_max=0.028),
    'B08':
    Band.create(name='B08',
                common_name='nir',
                description='Band 8 - NIR',
                center_wavelength=0.842,
                full_width_half_max=0.145),
    'B8A':
    Band.create(name='B8A',
                common_name='rededge',
                description='Band 8A - Vegetation red edge 4',
                center_wavelength=0.865,
                full_width_half_max=0.033),
    'B09':
    Band.create(name='B09',
                description='Band 9 - Water vapor',
                center_wavelength=0.945,
                full_width_half_max=0.026),
    'B11':
    Band.create(name='B11',
                common_name='swir16',
                description='Band 11 - SWIR (1.6)',
                center_wavelength=1.610,
                full_width_half_max=0.143),
    'B12':
    Band.create(name='B12',
                common_name='swir22',
                description='Band 12 - SWIR (2.2)',
                center_wavelength=2.190,
                full_width_half_max=0.242),
}

# A dict describing the resolutions that are
# available for each band as separate assets.
# The first resolution is the sensor gsd; others
# are downscaled versions.
BANDS_TO_RESOLUTIONS = {
    'B01': [
        60,
    ],
    'B02': [
        10,
        20,
        60,
    ],
    'B03': [
        10,
        20,
        60,
    ],
    'B04': [
        10,
        20,
        60,
    ],
    'B05': [
        20,
        20,
        60,
    ],
    'B06': [
        20,
        60,
    ],
    'B07': [
        20,
        60,
    ],
    'B08': [
        10,
    ],
    'B8A': [
        20,
        60,
    ],
    'B09': [60],
    'B11': [
        20,
        60,
    ],
    'B12': [
        20,
        60,
    ],
}
